==> bird-0.0.1%2B3.tar.gz: lib/src/layer3/smart_api/layer/service_endpoint/interfaces/service_endpoint.dart <==
import 'dart:async';

import 'package:bird/base.dart';
import 'package:bird/signal_wave.dart';
import 'package:bird/smart_api.dart';

/// A wrapper for a single data-type from an API.
///
/// This separates the way data is retrieved from a server from the actual
/// way how data is retrieved from a server.
abstract class ServiceEndpointStreams<T> implements BlocBase {
  /// Listeners of this stream will initiate what's necessary to get a steady stream of newest data.
  ///
  /// Call [restartObservable] when the last output was an error or [observeLastState] is [APIStateDisconnected].
  ///
  /// It will automatically restart when all listeners have stopped listening and a new listener starts listening for data.
  OEWave<T> get lastData;

  /// The underlying provider for [observeLast] can be in different states. Observe this to get the current state.
  ///
  /// Observing this state will not affect the logic behind [observeLast]. If there is no one listening
  /// to [observeLast] then this will emit [APIStateInitialized].
  Wave<APIState> get lastState;

  /// A service endpoint is stateful and will have to be disposed.
  ///
  /// An [APIService] will do that for you if its convenience methods are used.
  @override
  Future<void> dispose();
}

abstract class ServiceEndpoint<T> implements ServiceEndpointStreams<T> {
  /// Call this to restart [ServiceEndpointStreams.lastData] when its last output was an error
  /// (or [ServiceEndpointStreams.lastState] returned [APIStateDisconnected]).
  FutureOr<void> reconnect();

  ServiceEndpoint<R> map<R>(R Function(T) mappingFunction) {
    return _MappedServiceEndpoint<T, R>(this, mappingFunction);
  }
}

class _MappedServiceEndpoint<T, R> extends ServiceEndpoint<R> {
  final ServiceEndpoint<T> _source;
  final R Function(T) mappingFunction;

  _MappedServiceEndpoint(this._source, this.mappingFunction);

  @override
  Future<void> dispose() => _source.dispose();

  @override
  OEWave<R> get lastData => _source.lastData
      .map((a) => a.map((a) => a.map(mappingFunction)))
      .as(OEWave.from);

  @override
  Wave<APIState> get lastState => _source.lastState;

  @override
  FutureOr<void> reconnect() => _source.reconnect();
}

/// A [ServiceEndpoint] where data will be fetched once.
/// To receive new data [StaticServiceEndpoint.update] will have to be called.
// This acts like a pull based FRP system on top of the waves provided by [ServiceEndpointStreams].
// This could be potentially made simpler by adding a pull based FRP layer on top of the signal
// library. An ELM talk by Even C. talks about pull based systems being a subset of push based
// FRP systems.
abstract class StaticServiceEndpoint<T> implements ServiceEndpointStreams<T> {
  /// Updates data at [data] while [data] has listeners.
  Future<void> update();
}

==> bird-0.0.3%2B3.tar.gz: lib/src/layer3/smart_api/layer/service_endpoint/interfaces/service_endpoint.dart <==
import 'dart:async';

import 'package:bird/base.dart';
import 'package:bird/signal_wave.dart';
import 'package:bird/smart_api.dart';
import 'package:dartz/dartz.dart';

/// A wrapper for a single data-type from an API.
///
/// This separates the way data is retrieved from a server from the actual
/// way how data is retrieved from a server.
abstract class ServiceEndpointStreams<T> implements BlocBase {
  /// Listeners of this stream will initiate what's necessary to get a steady stream of newest data.
  ///
  /// Call [restartObservable] when the last output was an error or [observeLastState] is [APIStateDisconnected].
  ///
  /// It will automatically restart when all listeners have stopped listening and a new listener starts listening for data.
  Wave<Option<Either<AllErrors, T>>> get lastData;

  /// The underlying provider for [observeLast] can be in different states. Observe this to get the current state.
  ///
  /// Observing this state will not affect the logic behind [observeLast]. If there is no one listening
  /// to [observeLast] then this will emit [APIStateInitialized].
  Wave<APIState> get lastState;

  /// A service endpoint is stateful and will have to be disposed.
  ///
  /// An [APIService] will do that for you if its convenience methods are used.
  @override
  Future<void> dispose();
}

abstract class ServiceEndpoint<T> implements ServiceEndpointStreams<T> {
  /// Call this to restart [ServiceEndpointStreams.lastData] when its last output was an error
  /// (or [ServiceEndpointStreams.lastState] returned [APIStateDisconnected]).
  FutureOr<void> reconnect();

  ServiceEndpoint<R> map<R>(R Function(T) mappingFunction) {
    return _MappedServiceEndpoint<T, R>(this, mappingFunction);
  }
}

class _MappedServiceEndpoint<T, R> extends ServiceEndpoint<R> {
  final ServiceEndpoint<T> _source;
  final R Function(T) mappingFunction;

  _MappedServiceEndpoint(this._source, this.mappingFunction);

  @override
  Future<void> dispose() => _source.dispose();

  @override
  Wave<Option<Either<AllErrors, R>>> get lastData => _source.lastData
      .map((a) => a.map((a) => a.map(mappingFunction)));

  @override
  Wave<APIState> get lastState => _source.lastState;

  @override
  FutureOr<void> reconnect() => _source.reconnect();
}

/// A [ServiceEndpoint] where data will be fetched once.
/// To receive new data [StaticServiceEndpoint.update] will have to be called.
// This acts like a pull based FRP system on top of the waves provided by [ServiceEndpointStreams].
// This could be potentially made simpler by adding a pull based FRP layer on top of the signal
// library. An ELM talk by Even C. talks about pull based systems being a subset of push based
// FRP systems.
abstract class StaticServiceEndpoint<T> implements ServiceEndpointStreams<T> {
  /// Updates data at [data] while [data] has listeners.
  Future<void> update();
}
